package org.herac.tuxguitar_alsa.plugin;

import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Synthesizer;

import org.herac.tuxguitar.gui.TuxGuitar;
import org.herac.tuxguitar.gui.system.plugins.TGMidiPlugin;
import org.herac.tuxguitar.gui.system.plugins.TGPluginItem;
import org.herac.tuxguitar_alsa.plugin.synth.MidiReceiverImpl;
import org.herac.tuxguitar_alsa.plugin.synth.MidiSynthesizerImpl;


public class TGPluginImpl implements TGMidiPlugin{
	private TGPluginConfig config;
	private MidiSynthesizerImpl synthesizer;
	private TGPluginItemImpl item;
	private boolean enabled;
	
	public TGPluginImpl(){		
		this.config = new TGPluginConfig();
		this.enabled = this.config.getBooleanConfigValue("enabled",true);
		this.synthesizer = new MidiSynthesizerImpl(new MidiReceiverImpl(this.config.getIntConfigValue("device",0)));
		this.item = new TGPluginItemImpl(this);	
	}
	
	public void init(TuxGuitar instance) {		
		if(this.enabled){
			this.loadSynthesizer(getSynthesizer());
		}
	}
	
	public String getName() {
		return "Tuxguitar-alsa plugin";
	}

	public String getDescription() {
		return "Tuxguitar-alsa plugin";
	}

	public String getVersion() {
		return "test";
	}

	public String getVendor() {
		return "none";
	}

	public TGPluginItem getItem() {
		return this.item;
	}
	
	public MidiSynthesizerImpl getSynthesizer(){
		return this.synthesizer;
	}
	
	public boolean isEnabled(){
		return this.enabled;
	}
	
	public void changeConfig(int device,boolean enabled){
		try {
			this.config.setProperty("device",device);
			this.config.setProperty("enabled",enabled);
			this.config.save();
			if(enabled){
				if(!this.enabled){
					this.loadSynthesizer(getSynthesizer());
				}
				if(device != getSynthesizer().getReceiver().getDevice()){
					getSynthesizer().getReceiver().open(device);
				}
			}else{
				if(this.enabled){
					this.loadSynthesizer(null);
				}
			}			
			this.update();
			this.enabled = enabled;
		} catch (MidiUnavailableException e) {
			e.printStackTrace();
		}
	}
		
	private void update(){
		TuxGuitar.instance().loadProperties();
		if(TuxGuitar.instance().getPlayer().isRunning()){
			TuxGuitar.instance().getPlayer().updatePrograms();
			TuxGuitar.instance().getPlayer().updateControllers();
		}
	}
	
	public void close() {
		synthesizer.close();
	}

	private void loadSynthesizer(Synthesizer synthesizer){
		if(TuxGuitar.instance().getPlayer() == null){		
			TuxGuitar.instance().initPlayer(false);
		}
		if(TuxGuitar.instance().getPlayer() instanceof org.herac.tuxguitar.player.impl.MidiPlayerImpl){
			//asigna el nuevo synthesizer
			TuxGuitar.instance().getPlayer().setSynthesizer(synthesizer);
		}
	}
}
